/*global jQuery, Drupal, drupalSettings, window*/
/*jslint white:true, multivar, this, browser:true*/

(function($, Drupal, drupalSettings, window)
{
  "use strict";

  var initialized, usernameInput, container, inputWrapper, autocompleteUsernames, autocompleteResultsContainer, insertedUsernames, usernameInputTimeout;

  function triggerCommands(data) {
    var ajaxObject = Drupal.ajax({
      url: "",
      base: false,
      element: false,
      progress: false
    });

    // Trigger any any ajax commands in the response
    ajaxObject.success(data, "success");
  }

  function insertUsernameOnComplete(inputCount, username) {
    var inputs = inputWrapper.find("input[type='text']");
    if (inputs.length > inputCount) {
      if (inputs.last().val() === "") {
        inputs.last().val(username);
        usernameInput.focus();
      }
      else {
        window.setTimeout(function() {
          insertUsernameOnComplete(inputCount + 1, username);
        }, 50);
      }
    }
    else {
      window.setTimeout(function() {
        insertUsernameOnComplete(inputCount, username);
      }, 50);
    }
  }

  function addUserToMembers(username, validateName) {
    var found, trimmedVal;

    username = $.trim(username);
    if(username.length) {
      insertedUsernames[username] = true;

      Drupal.theme("usernameDisplayItem", username).insertBefore(usernameInput);

      inputWrapper.find("input[type='text']").each(function() {
        trimmedVal = $.trim($(this).val());
        if (trimmedVal === "") {
          $(this).val(username);
          found = true;
        }
        else if (trimmedVal === username) {
          found = true;
        }

        if (found) {
          return false;
        }
      });

      if (!found) {
        insertUsernameOnComplete(inputWrapper.find("input[type='text']").length, username);
        $("#private-message-add-form .form-submit").mousedown();
      }

      if (validateName) {
        $.ajax({
          url:drupalSettings.privateMessageMembersWidget.validateUsernameUrl,
          data:{username:username},
          success:function(data) {
            triggerCommands(data);
          }
        });
      }
    }
  }

  function markUsernameInvalid(username) {
    container.find(".private-message-member-display-item").each(function() {
      if($(this).attr("data-username") === username) {
        $(this).addClass("invalid-username");
      }
    });
  }

  function hideAutocompleteResults() {
    if (autocompleteResultsContainer) {
      Drupal.detachBehaviors(autocompleteResultsContainer);
      autocompleteResultsContainer.remove();
      autocompleteResultsContainer = null;
    }
  }

  function showAutocompleteResults(string) {
    hideAutocompleteResults();

    if (autocompleteUsernames && autocompleteUsernames[string] && autocompleteUsernames[string].length) {
      var position, list, i;

      position = usernameInput.position();
      autocompleteResultsContainer =  $("<div/>", {id:"pm-members-autocomplete-results-wrapper"}).css({top:(position.top + usernameInput.outerHeight() - 1) + "px", left:position.left + "px", width:usernameInput.outerWidth() + "px"});

      list = $("<ul/>", {class:"ui-front ui-menu ui-widget ui-widget-content"});

      i = 1;
      $.each(autocompleteUsernames[string], function(uid) {
        var username = autocompleteUsernames[string][uid].username;
        if (!insertedUsernames[username]) {
          $("<li/>", {class:"ui-menu-item"}).append($("<a/>", {class:"pm-autocomplete-search-result", "data-username":username, tabindex:i}).text(username)).appendTo(list);
          i += 1;
        }
      });

      if (list.children("li:first").length) {
        autocompleteResultsContainer.append(list).appendTo(container);
        Drupal.attachBehaviors(autocompleteResultsContainer);
      }
    }
  }

  function membersFieldListener() {
    usernameInput.once("members-field-listener").each(function() {
      $(this).keydown(function(e) {
        var keyCode;

        keyCode = e.keyCode || e.which;

        // Tab key
        if (keyCode === 9) {
          if (usernameInput.val().length) {
            e.preventDefault();
            addUserToMembers(usernameInput.val(), true);
            usernameInput.val("");
            hideAutocompleteResults();
          }
        }
        // Down key pressed
        else if (keyCode === 40) {
          if (autocompleteResultsContainer) {
            e.preventDefault();
            autocompleteResultsContainer.find("a:first").focus();
          }
        }
        // Backspace key
        else if (keyCode === 8 && !$(this).val().length) {
          $(this).parent().children(".private-message-member-display-item:last").children(".pm-username-remove-link:last").click();
        }
      })
      .keyup(function(e) {
        var keyCode;

        keyCode = e.keyCode || e.which;

        // Alphabet key or backspace when there is a value
        if ((keyCode >= 65 && keyCode <= 90) || (keyCode === 8 && $(this).val().length)) {
          window.setTimeout(function() {
            var username = usernameInput.val();

            if (username.length) {
              if (!autocompleteUsernames[username]) {
                $.ajax({
                  url:drupalSettings.privateMessageMembersWidget.callbackPath,
                  data:{username:username},
                  success:function(data) {
                    triggerCommands(data);
                  }
                });
              }
              else {
                showAutocompleteResults(username);
              }
            }
            else {
              hideAutocompleteResults(username);
            }
          }, 0);
        }
      })
      .blur(function() {
        usernameInputTimeout = window.setTimeout(function() {
          if(usernameInput.val().length) {
            addUserToMembers(usernameInput.val(), true);
            usernameInput.val("");
          }
          hideAutocompleteResults();
        }, 20);
      });
    });
  }

  function toRegionListener() {
    $(".form-item-pm-to-display").once("to-region-listener", function() {
      $(this).click(function() {
        $(this).find("input:first").focus();
      });
    });
  }


  function bodyListener(context) {
    $(context).find("body").once("body-listener").each(function() {
      $(this).click(function() {
        hideAutocompleteResults();
      });
    });
  }

  function autocompleteListener() {
    if (autocompleteResultsContainer) {
      autocompleteResultsContainer.children("ul:first").children("li").children("a").once("pm-autocomplete-listener").each(function() {

        $(this).click(function(e) {
          e.preventDefault();
          e.stopPropagation();

          addUserToMembers($(this).attr("data-username"));
          hideAutocompleteResults();
          usernameInput.val("").focus();
        })
        .focus(function (){
          window.clearTimeout(usernameInputTimeout);
        })
        .keydown(function(e) {
          e.preventDefault();

          var keyCode;

          keyCode = e.keyCode || e.which;

          // Down key
          if (keyCode === 40) {
            if ($(this).parent().next("li").length) {
              $(this).parent().next("li").children(":first").focus();
            }
            else {
              e.preventDefault();
            }
          }
          // Up key
          else if (keyCode === 38) {
            if ($(this).parent().prev("li").length) {
              $(this).parent().prev("li").children(":first").focus();
            }
            else {
              usernameInput.focus();
            }
          }
          // Tab key or Enter key
          else if (keyCode === 9 || keyCode === 13) {
            e.preventDefault();
            e.stopPropagation();

            addUserToMembers($(this).attr("data-username"));
            hideAutocompleteResults();
            usernameInput.val("").focus();
          }
        });
      });
    }
  }

  function init() {
    if (!initialized) {

      var label;

      initialized = true;
      usernameInput = $("<input/>", {type:"text", id:"thread-members-input", placeholder:drupalSettings.privateMessageMembersWidget.placeholder}).attr("autocomplete", "off").attr("size", drupalSettings.privateMessageMembersWidget.fieldSize);
      label = $("<label/>", {for:"thread-members-input"}).text("To:");
      inputWrapper = $(".field--widget-private-message-thread-member-widget:first").hide();
      container = $("<div/>", {id:"thread-members-display-container"}).append(label).append(usernameInput).insertAfter(inputWrapper);
      autocompleteUsernames = {};
      insertedUsernames = {};
      inputWrapper.find("input[type='text']").each(function() {
        var trimmedVal = $.trim($(this).val());
        if (trimmedVal.length) {
          addUserToMembers(trimmedVal, true);
        }
      });

      usernameInput.focus();
    }
  }

  Drupal.behaviors.privateMessageMembersWidget = {
    attach:function(context) {
      init();
      membersFieldListener();
      bodyListener(context);
      autocompleteListener();
      toRegionListener();

      Drupal.AjaxCommands.prototype.privateMessageMembersAutocompleteResponse = function(ajax, response) {
        // stifles jSlint warning.
        ajax = ajax;

        autocompleteUsernames[response.string] = response.userInfo;

        showAutocompleteResults(response.string);
      };


      Drupal.AjaxCommands.prototype.privateMessageMemberUsernameValidated = function(ajax, response) {
        // stifles jSlint warning.
        ajax = ajax;

        if (!response.validUsername) {
          markUsernameInvalid(response.username);
        }
      };
	}
  };

  Drupal.theme.usernameDisplayItem = function (userName) {
    return $("<div />", {class:"private-message-member-display-item", "data-username":userName}).append($("<span/>", {class:"pm-username", "data-pm-username":userName}).text(userName)).append($("<span/>", {class:"pm-username-remove-link"}).text("X").click(function() {
	    var username = $(this).parent().children(".pm-username:first").attr("data-pm-username");

	    delete insertedUsernames[username];

	    inputWrapper.find("input[type='text']").each(function() {
          if ($(this).val() === username) {
            $(this).val("");
          }
        });

        $(this).parent().remove();
    }));
  };

}(jQuery, Drupal, drupalSettings, window));
